@@ -156,9 +156,7 @@ The evaluation of PROPERTIES to produce a property application specification
 may retrieve existing hostattrs, but should not set any new ones (not to be
 confused with how the :HOSTATTRS subroutines of properties in PROPERTIES may
 set additional hostattrs)."
-  (once-only ((host (if (stringp host)
-                        `(make-host :hostattrs (list :hostname (list ,host)))
-                        host)))
+  (once-only ((host `(ensure-host ,host)))
     `(deploy-these* ',connections
                     ,host
                     (let ((*host* (shallow-copy-host ,host)))
@@ -272,7 +270,8 @@ PROPERTIES, like DEPLOY-THESE."
   (:desc (declare (ignore connections host properties)) "Subdeployment")
   (:preprocess
    (list (preprocess-connections connections)
-         (preprocess-host (replace-propspec-into-host host properties))))
+         (preprocess-host (replace-propspec-into-host host properties))
+         nil))
   (:hostattrs
    (declare (ignore connections properties))
    (%propagate-hostattrs host))
@@ -305,7 +304,7 @@ different user."
      (setf (getf (slot-value host 'hostattrs) :data) nil)
      (setq host (preprocess-host host))
      (doplist (k v (hostattrs host))
-              (loop with root = (getf (hostattrs *host*) k)
+              (loop with root = (get-hostattrs k)
                     for cell on v until (eq cell root)
                     collect (car cell) into accum
                     finally (apply #'push-hostattrs k (nreverse accum))))
@@ -323,7 +322,7 @@ different user."
 (defun %propagate-hostattrs (host)
   (dolist (system (propspec-systems (host-propspec host)))
     (pushnew system (slot-value (host-propspec *host*) 'systems)))
-  (dolist (attr (getf (hostattrs host) :data))
+  (dolist (attr (get-hostattrs :data host))
     (push-hostattrs :data attr)))
 
 (defprop evals :posix (&rest forms)
