@@ -65,8 +65,11 @@
            #:compile-file-pathname*
 
            ;; util.lisp
+           #:multiple-value-mapcan
            #:lines
            #:unlines
+           #:words
+           #:unwords
            #:noop
            #:symbol-named
            #:memstring=
@@ -151,6 +154,7 @@
            #:push-hostattrs
            #:pushnew-hostattrs
            #:get-hostname
+           #:get-short-hostname
            #:require-data
            #:failed-change
            #:assert-euid-root
@@ -180,6 +184,7 @@
            #:desc
            #:on-change
 	   #:as
+           #:with-flagfile
 
            ;; host.lisp
            #:host
@@ -192,6 +197,7 @@
            #:hostattrs
            #:host-propspec
            #:preprocess-host
+           #:ensure-host
            #:with-preserve-hostattrs
            #:with-replace-hostattrs
 
@@ -316,8 +322,10 @@
 (defpackage :consfigurator.property.mount
   (:use #:cl #:alexandria #:consfigurator)
   (:local-nicknames (#:os    #:consfigurator.property.os)
-                    (#:cmd   #:consfigurator.property.cmd))
-  (:export #:unmounted-below
+                    (#:cmd   #:consfigurator.property.cmd)
+                    (#:file  #:consfigurator.property.file))
+  (:export #:mounted
+           #:unmounted-below
            #:unmounted-below-and-removed))
 
 (defpackage :consfigurator.property.service
@@ -362,6 +370,7 @@
   (:use #:cl #:consfigurator)
   (:local-nicknames (#:file  #:consfigurator.property.file))
   (:export #:has-account
+           #:has-groups
 	   #:has-login-shell
            #:has-enabled-password
 	   #:passwd-entry))
@@ -424,7 +433,6 @@
            #:opened-partition
 
            #:lvm-volume-group
-           #:activated-lvm-volume-group
            #:lvm-logical-volume
            #:activated-lvm-logical-volume
            #:lvm-physical-volume
@@ -451,7 +459,9 @@
            #:caches-cleaned
            #:raw-image-built-for
            #:host-volumes-created
+           #:host-logical-volumes-exist
 
+           #:parse-volume-size
            #:volumes))
 
 (defpackage :consfigurator.property.fstab
@@ -459,6 +469,7 @@
   (:local-nicknames (#:os    #:consfigurator.property.os)
                     (#:file  #:consfigurator.property.file))
   (:export #:volume->entry
+           #:entries
            #:entries-for-volumes
            #:entries-for-opened-volumes))
 
@@ -479,11 +490,6 @@
   (:local-nicknames (#:file      #:consfigurator.property.file))
   (:export #:snapshot-extracted))
 
-(defpackage :consfigurator.property.ssh
-  (:use #:cl #:consfigurator)
-  (:local-nicknames (#:file      #:consfigurator.property.file))
-  (:export #:authorized-keys))
-
 (defpackage :consfigurator.property.sshd
   (:use #:cl #:consfigurator)
   (:local-nicknames (#:re        #:cl-ppcre)
@@ -492,7 +498,19 @@
                     (#:apt       #:consfigurator.property.apt))
   (:export #:installed
            #:configured
-           #:no-passwords))
+           #:no-passwords
+           #:get-host-public-keys
+           #:has-host-public-key
+           #:has-host-key))
+
+(defpackage :consfigurator.property.ssh
+  (:use #:cl #:alexandria #:consfigurator)
+  (:local-nicknames (#:file      #:consfigurator.property.file)
+                    (#:sshd      #:consfigurator.property.sshd))
+  (:export #:authorized-keys
+           #:known-host
+           #:globally-known-host
+           #:parent-is-globally-known-host))
 
 (defpackage :consfigurator.property.locale
   (:use #:cl #:consfigurator)
@@ -540,10 +558,35 @@
   (:local-nicknames (#:cmd       #:consfigurator.property.cmd)
                     (#:container #:consfigurator.property.container)
                     (#:file      #:consfigurator.property.file))
-  (:export #:configured
+  (:export #:is
+           #:configured
            #:mailname-configured
            #:search-configured))
 
+(defpackage :consfigurator.property.network
+  (:use #:cl #:consfigurator)
+  (:local-nicknames (#:os        #:consfigurator.property.os)
+                    (#:file      #:consfigurator.property.file))
+  (:export #:static))
+
+(defpackage :consfigurator.property.libvirt
+  (:use #:cl #:alexandria #:consfigurator)
+  (:local-nicknames (#:os        #:consfigurator.property.os)
+                    (#:cmd       #:consfigurator.property.cmd)
+                    (#:file      #:consfigurator.property.file)
+                    (#:chroot    #:consfigurator.property.chroot)
+                    (#:apt       #:consfigurator.property.apt))
+  (:export #:installed
+           #:default-network-started
+           #:default-network-autostarted
+           #:defined
+           #:started
+           #:when-started
+           #:kvm-boots-chroot-for
+           #:kvm-boots-chroot-for.
+           #:kvm-boots-chroot
+           #:kvm-boots-chroot.))
+
 (defpackage :consfigurator.connection.local
   (:use #:cl #:consfigurator #:alexandria)
   (:export #:local-connection))
