@@ -108,14 +108,26 @@
 
 (defun propunapply (prop &rest args)
   (with-some-errors-are-failed-change
-    (let ((check (get prop 'check)))
-      (if (and check (not (apply check args)))
-          :no-change
-          (apply (get prop 'unapply (constantly :no-change)) args)))))
+    (let ((check (get prop 'check))
+          (apply (get prop 'apply))
+          (unapply (get prop 'unapply)))
+      ;; Only fail if there's no :UNAPPLY when there is an :APPLY, because
+      ;; that is the case in which we can't do what was requested.  If there
+      ;; is no :APPLY then we can infer that there is nothing on the host to
+      ;; unapply (this will be the case for pure :HOSTATTRS properties).
+      (cond
+        ((or (and (not apply) (not unapply))
+             (and check (not (apply check args))))
+         :no-change)
+        (unapply
+         (apply unapply args))
+        (apply
+         (failed-change
+"Attempt to unapply property with :APPLY subroutine but no :UNAPPLY subroutine."))))))
 
 (defun propappunapply (propapp)
   (if propapp
-      (apply #'propappunapply propapp)
+      (apply #'propunapply propapp)
       :no-change))
 
 (defvar *known-properties* nil
@@ -311,7 +323,8 @@ Use DEFPROPLIST/DEFPROPSPEC to avoid trouble."))
 subroutine, muffle warnings about calling a property with a :HOSTATTRS
 subroutine programmatically.  Use this only when you know that the :HOSTATTRS
 subroutine does not push any new hostattrs."
-  (unless (and (listp form) (isprop (car form)))
+  (unless
+      (and (listp form) (or (not (fboundp (car form))) (isprop (car form))))
     (simple-program-error "~A is not a programmatic call to a property." form))
   `(handler-bind ((programmatic-apply-hostattrs #'muffle-warning))
      ,form))
@@ -333,7 +346,8 @@ subroutine does not push any new hostattrs."
                              `((declare
                                 (ignorable
                                  ,@(ordinary-ll-variable-names
-                                    (ordinary-ll-without-&aux lambda))))))
+                                    (ordinary-ll-without-&aux lambda)
+                                    :include-supplied-p t)))))
                       ,@slot)))))
 
 (define-property-defining-macro defpropspec (type lambda slots forms)
@@ -393,7 +407,8 @@ You can usually use DEFPROPLIST instead of DEFPROPSPEC, which see."
                       ,@(and (member (caar forms) '(:desc :hostattrs))
                              `((declare
                                 (ignorable
-                                 ,@(ordinary-ll-variable-names lambda)))))
+                                 ,@(ordinary-ll-variable-names
+                                    lambda :include-supplied-p t)))))
 		      ,@(cdr (pop forms))))))
   (setf (getf slots :hostattrs)
         `(lambda (plist)
@@ -451,20 +466,36 @@ other than constant values and propapps to property combinators."
 this property cannot be applied to this host.  E.g. the property will try to
 install an apt package but the host is FreeBSD.")
 
-(defun get-hostattrs (k)
+(defparameter *preprocessing-host* nil
+  "HOST value currently being preprocessed.
+Used by GET-HOSTATTRS to break infinite loops.")
+
+(defun get-hostattrs (k &optional (host *host*) &aux (host (ensure-host host)))
   "Retrieve the list of static informational attributes of type KEY.
 
 Called by property :HOSTATTRS, :APPLY and :UNAPPLY subroutines."
-  (getf (slot-value *host* 'hostattrs) k))
+  ;; Ensure the host is preprocessed so the desired hostattrs are actually
+  ;; there, assuming we're not already preprocessing it.  Avoid calling
+  ;; PREPROCESS-HOST on PREPROCESSED-HOST values to avoid pointless copying.
+  ;;
+  ;; This is just to improve readability for some property definitions and
+  ;; avoid confusing situations where hostattrs appear to be missing (for
+  ;; example, if the hostname is not set until HOSTNAME:IS); properties which
+  ;; will look up multiple hostattrs by supplying a value for HOST should call
+  ;; PREPROCESS-HOST on that value themselves.
+  (let ((host (if (and (subtypep (class-of host) 'unpreprocessed-host)
+                       (not (eql host *preprocessing-host*)))
+                  (preprocess-host host) host)))
+    (getf (slot-value host 'hostattrs) k)))
 
-(defun get-hostattrs-car (k)
-  (car (get-hostattrs k)))
+(defun get-hostattrs-car (k &optional (host *host*))
+  (car (get-hostattrs k host)))
 
-(defun get-parent-hostattrs (k)
-  (getf (get-hostattrs :parent-hostattrs) k))
+(defun get-parent-hostattrs (k &optional (host *host*))
+  (getf (get-hostattrs :parent-hostattrs host) k))
 
-(defun get-parent-hostattrs-car (k)
-  (car (get-parent-hostattrs k)))
+(defun get-parent-hostattrs-car (k &optional (host *host*))
+  (car (get-parent-hostattrs k host)))
 
 (defun push-hostattrs (k &rest vs)
   "Push new static informational attributes VS of type KEY.
@@ -487,11 +518,19 @@ is needed to deploy a property.
 Called by property :HOSTATTRS subroutines."
   (pushnew-hostattrs :data (cons iden1 iden2)))
 
-(defun get-hostname ()
-  "Get the hostname of the host to which properties are being applied.
+(defun get-hostname (&optional (host *host*))
+  "Get the hostname of HOST, defaulting to the host to which properties are
+being applied.
 
 Called by property subroutines."
-  (get-hostattrs-car :hostname))
+  (get-hostattrs-car :hostname host))
+
+(defun get-short-hostname (&optional (host *host*))
+  "Get the short hostname of HOST, defaulting to the host to which properties
+are being applied.
+
+Called by property subroutines."
+  (car (split-string (get-hostattrs-car :hostname host) :separator ".")))
 
 
 ;;;; :APPLY subroutines
