@@ -91,16 +91,21 @@ subsequently replace the copied values of some slots.")
 (defgeneric subvolumes-of-type (type volume)
   (:documentation
    "Recursively examine VOLUME and its VOLUME-CONTENTS and return a list of all
-volumes encountered whose type is a subtype of TYPE.")
+volumes encountered whose type is a subtype of TYPE.
+Returns as a second value a corresponding list of the immediate parents of
+each returned volume.")
   (:method ((type symbol) (volume volume))
-    (labels ((walk (volume)
-               (let ((contents
-                       (and (slot-boundp volume 'volume-contents)
-                            (mapcan #'walk
-                                    (ensure-cons (volume-contents volume))))))
+    (labels ((walk (volume parent &aux (second-arg (list volume)))
+               (multiple-value-bind (contents contents-parents)
+                   (and (slot-boundp volume 'volume-contents)
+                        (multiple-value-mapcan
+                         #'walk (ensure-cons (volume-contents volume))
+                         (rplacd second-arg second-arg)))
                  (if (subtypep (type-of volume) type)
-                     (cons volume contents) contents))))
-      (walk volume))))
+                     (values (cons volume contents)
+                             (cons parent contents-parents))
+                     (values contents contents-parents)))))
+      (walk volume nil))))
 
 (defgeneric all-subvolumes (volume)
   (:documentation
@@ -199,10 +204,7 @@ Return values, if any, should be ignored.")
    "A VOLUME object which has been made directly accessible as a block device."))
 
 (defmethod open-volume ((volume opened-volume) file)
-  volume)
-
-(defmethod create-volume ((volume opened-volume) file)
-  (values))
+  (copy-volume-and-contents volume))
 
 (defgeneric make-opened-volume (volume device-file)
   (:documentation
@@ -405,77 +407,37 @@ The default value of 0 means the next free sector.")
 
 ;;;; LVM
 
-(defclass lvm-volume-group (top-level-volume)
-  ((volume-label
-    :documentation "The name of the VG, often starting with \"vg_\".")
-   ;; (volume-depth :initform 3)
-   (volume-contents
-    :type list
-    :documentation "A list of objects of type LVM-LOGICAL-VOLUME."))
-  (:documentation
-   "An LVM volume group.  Typically specified as a top level volume in
-DISK:HAS-VOLUMES, rather than as the VOLUME-CONTENTS of another volume."))
-
-(defclass-opened-volume activated-lvm-volume-group (lvm-volume-group)
-                        :device-file-type null)
-
-(defun all-vgs ()
-  (mapcar (curry #'string-trim " ")
-          (runlines "vgs" "--no-headings" "-ovg_name")))
-
-(defmethod open-volume ((volume lvm-volume-group) (file null))
-  (with-slots (volume-label volume-contents) volume
-    (mrun "vgscan")
-    (mrun "vgchange" "-ay" volume-label)
-    ;; lvm(8) says not to use the /dev/mapper names, but rather /dev/vg/lv
-    (let ((dir (ensure-directory-pathname
-                (merge-pathnames volume-label #P"/dev/"))))
-      (values
-       (make-opened-volume volume nil)
-       (loop for lv in volume-contents
-             collect (make-opened-volume
-                      lv (merge-pathnames (volume-label lv) dir)))))))
-
-(defmethod close-volume ((volume activated-lvm-volume-group))
-  (with-slots (volume-label) volume
-    (mrun "vgchange" "-an" volume-label)))
-
-(defmethod create-volume ((volume lvm-volume-group) (file null))
-  ;; We expect that the VG was already created when the PVs were created, and
-  ;; this method actually implements creation of the LVs.
-  (with-slots (volume-label volume-contents) volume
-    (unless (member volume-label (all-vgs) :test #'string=)
-      (failed-change "Looks like no PVs for VG ~A?" volume-label))
-    (dolist (lv volume-contents)
-      (mrun :inform "lvcreate"
-            (if (and (slot-boundp lv 'volume-size)
-                     (eql (volume-size lv) :remaining))
-                '("-l" "100%FREE")
-                `("-L" ,(format nil "~DM" (volume-minimum-size lv))))
-            volume-label
-            "-n" (volume-label lv)))))
-
-(defclass lvm-logical-volume (volume)
-  ((volume-label
-    :initform (simple-program-error "LVs must have names.")
-    :documentation "The name of the LV, often starting with \"lv_\".")))
-
-(defclass-opened-volume activated-lvm-logical-volume (lvm-logical-volume))
-
 (defclass lvm-physical-volume (volume)
-  ((volume-group
+  ((lvm-volume-group
     :type string
     :initarg :volume-group
     :initform
     (simple-program-error "LVM physical volume must have volume group.")
-    :accessor volume-group
+    :accessor lvm-volume-group
     :documentation
     "The name of the LVM volume group to which this volume belongs.")
+
+   ;; pvcreate(8) options
    (data-alignment
     :type string
     :initarg :data-alignment
     :accessor data-alignment
-    :documentation "Value for the --dataalignment argument to pvcreate(8)."))
+    :documentation "Value for the --dataalignment argument to pvcreate(8).")
+
+   ;; vgcreate(8) options
+   (physical-extent-size
+    :type string
+    :initarg :physical-extent-size
+    :accessor physical-extent-size
+    :documentation "Value for the --dataalignment argument to vgcreate(8).
+Should be the same for all PVs in this VG.")
+   (alloc
+    :type string
+    :initarg :alloc
+    :accessor alloc
+    :documentation "Value for the --alloc argument to vgcreate(8).
+Should be the same for all PVs in this VG."))
+
   (:documentation "An LVM physical volume.
 We do not specify what logical volumes it contains."))
 
@@ -489,9 +451,96 @@ We do not specify what logical volumes it contains."))
         (and (slot-boundp volume 'data-alignment)
              `("--dataalignment" ,(data-alignment volume)))
         file)
-  (if (member (volume-group volume) (all-vgs) :test #'string=)
-      (mrun :inform "vgextend" (volume-group volume) file)
-      (mrun :inform "vgcreate" "--systemid" "" (volume-group volume) file)))
+  (if (member (lvm-volume-group volume) (all-vgs) :test #'string=)
+      (mrun :inform "vgextend" (lvm-volume-group volume) file)
+      (mrun :inform "vgcreate" "--systemid" ""
+            (and (slot-boundp volume 'physical-extent-size)
+                 `("--physicalextentsize" ,(physical-extent-size volume)))
+            (and (slot-boundp volume 'alloc)
+                 `("--alloc" ,(alloc volume)))
+            (lvm-volume-group volume) file)))
+
+(defun all-vgs ()
+  (mapcar (curry #'string-trim " ")
+          (runlines "vgs" "--no-headings" "-ovg_name")))
+
+(defclass lvm-logical-volume (top-level-volume)
+  ((volume-label
+    :initform (simple-program-error "LVs must have names.")
+    :documentation "The name of the LV, often starting with \"lv_\".")
+   (lvm-volume-group
+    :type string
+    :initarg :volume-group
+    :initform
+    (simple-program-error "LVM logical volumes must have a volume group.")
+    :accessor lvm-volume-group
+    :documentation
+    "The name of the LVM volume group to which this volume belongs.")))
+
+(defclass-opened-volume activated-lvm-logical-volume (lvm-logical-volume))
+
+(defmethod volume-contents-minimum-size ((volume lvm-logical-volume))
+  "LVs cannot be of zero size."
+  (max 1 (call-next-method)))
+
+(defmethod open-volume ((volume lvm-logical-volume) (file null))
+  (with-slots (volume-label lvm-volume-group) volume
+    (mrun "lvchange" "-ay" (strcat lvm-volume-group "/" volume-label))
+    ;; lvm(8) says not to use the /dev/mapper names, but rather /dev/vg/lv
+    (make-opened-volume
+     volume (merge-pathnames volume-label
+                             (ensure-directory-pathname
+                              (merge-pathnames lvm-volume-group #P"/dev/"))))))
+
+(defmethod close-volume ((volume activated-lvm-logical-volume))
+  (mrun "lvchange" "-an"
+        (strcat (lvm-volume-group volume) "/" (volume-label volume))))
+
+(defmethod create-volume ((volume lvm-logical-volume) (file null))
+  (with-slots (volume-label lvm-volume-group) volume
+    ;; Check that the VG exists.
+    (unless (member lvm-volume-group (all-vgs) :test #'string=)
+      (failed-change "Looks like no PVs for VG ~A?" lvm-volume-group))
+    ;; Create the LV.
+    (mrun :inform "lvcreate" "-Wn"
+          (if (and (slot-boundp volume 'volume-size)
+                   (eql (volume-size volume) :remaining))
+              '("-l" "100%FREE")
+              `("-L" ,(format nil "~DM" (volume-minimum-size volume))))
+          lvm-volume-group "-n" volume-label)))
+
+(defprop host-lvm-logical-volumes-exist :lisp ()
+  (:desc "Host LVM logical volumes all exist")
+  (:hostattrs (os:required 'os:linux))
+  (:apply
+   (assert-euid-root)
+   (let* ((existing-lvs
+            (loop for (lv vg) in (mapcar #'words (cdr (runlines "lvs")))
+                  collect (cons lv vg)))
+          (to-create
+            ;; LVs are TOP-LEVEL-VOLUMEs.
+            (loop for volume in (get-hostattrs :volumes)
+                  when (subtypep (class-of volume) 'lvm-logical-volume)
+                    unless (member (cons (volume-label volume)
+                                         (lvm-volume-group volume))
+                                   existing-lvs :test #'equal)
+                      collect volume))
+          (to-mount
+            (loop for volume in to-create
+                  nconc (loop for volume
+                                in (subvolumes-of-type 'filesystem volume)
+                              when (slot-boundp volume 'mount-point)
+                                collect volume))))
+     (if to-create
+         (prog2 (ignoring-hostattrs
+                 (consfigurator.property.fstab:entries-for-volumes to-create))
+             (create-volumes-and-contents to-create)
+           (dolist (volume to-create)
+             (open-volume volume nil))
+           (dolist (volume to-mount)
+             (consfigurator.property.mount:mounted
+              :target (mount-point volume))))
+         :no-change))))
 
 
 ;;;; Filesystems
@@ -632,8 +681,13 @@ single attempt will be made to close all volumes opened up to that point."
                    (multiple-value-bind (opened opened-contents)
                        (open-volume volume file)
                      (setq opened-volumes
-                           (append opened-contents
-                                   (cons opened opened-volumes)))
+                           (append
+                            opened-contents
+                            ;; Don't queue for closure volumes which were
+                            ;; already open before we began.
+                            (if (subtypep (class-of volume) 'opened-volume)
+                                opened-volumes
+                                (cons opened opened-volumes))))
                      (dolist (opened-volume (or opened-contents `(,opened)))
                        (when (slot-boundp opened-volume 'volume-contents)
                          (open-volume-and-contents
@@ -713,34 +767,42 @@ must not be modified."
          (mapc #'close-volume (get-connattr :opened-volumes)))))
    :args (cdr propapp)))
 
-(defgeneric create-volumes-and-contents (volumes)
-  (:documentation
-   "Where each of VOLUMES is a VOLUME which may be created by calling
-CREATE-VOLUME with NIL as the second argument, recursively create each of
-VOLUMES and any contents thereof.
-**THIS METHOD UNCONDITIONALLY FORMATS DISKS, POTENTIALLY DESTROYING DATA.**")
-  (:method (volumes)
-    (let (opened-volumes)
-      (labels
-          ((create-volume-and-contents (volume file)
-             (create-volume volume file)
-             (when (slot-boundp volume 'volume-contents)
-               (multiple-value-bind (opened opened-contents)
-                   (open-volume volume file)
-                 (setq opened-volumes
-                       (append opened-contents (cons opened opened-volumes)))
-                 (if opened-contents
-                     (dolist (opened-volume opened-contents)
-                       (when (slot-boundp opened-volume 'volume-contents)
-                         (create-volume-and-contents
-                          (volume-contents opened-volume)
-                          (device-file opened-volume))))
-                     (create-volume-and-contents
-                      (volume-contents opened) (device-file opened)))))))
-        (unwind-protect
-             (mapc (rcurry #'create-volume-and-contents nil) volumes)
-          (mrun "sync")
-          (mapc #'close-volume opened-volumes))))))
+(defun create-volumes-and-contents (volumes &optional files)
+  "Where each of VOLUMES is a VOLUME which may be created by calling
+CREATE-VOLUME with the corresponding entry of FILES, or NIL, as a second
+argument, recursively create each of VOLUMES and any contents thereof.
+**THIS FUNCTION UNCONDITIONALLY FORMATS DISKS, POTENTIALLY DESTROYING DATA.**"
+  (let (opened-volumes)
+    (labels
+        ((create-volume-and-contents
+             (volume file
+              &aux
+                (already-opened (subtypep (class-of volume) 'opened-volume)))
+           (unless already-opened
+             (create-volume volume file))
+           (when (slot-boundp volume 'volume-contents)
+             (multiple-value-bind (opened opened-contents)
+                 (open-volume volume file)
+               (setq opened-volumes
+                     (append opened-contents
+                             ;; Don't queue for closure volumes which were
+                             ;; already open before we began.
+                             (if already-opened
+                                 opened-volumes
+                                 (cons opened opened-volumes))))
+               (if opened-contents
+                   (dolist (opened-volume opened-contents)
+                     (when (slot-boundp opened-volume 'volume-contents)
+                       (create-volume-and-contents
+                        (volume-contents opened-volume)
+                        (device-file opened-volume))))
+                   (create-volume-and-contents
+                    (volume-contents opened) (device-file opened)))))))
+      (unwind-protect
+           (mapc #'create-volume-and-contents
+                 volumes (loop repeat (length volumes) collect (pop files)))
+        (mrun "sync")
+        (mapc #'close-volume opened-volumes)))))
 
 
 ;;;; Properties
@@ -851,15 +913,13 @@ least the following:
 Unless REBUILD, the image will not be repartitioned even if the specification
 of the host's volumes changes, although the contents of the image's
 filesystems will be incrementally updated when other properties change."
-  (:desc (declare (ignore options rebuild))
-         (let ((hostname (car (getf (hostattrs host) :hostname))))
-           #?"Built image for ${hostname} @ ${image-pathname}"))
+  (:desc #?"Built image for ${(get-hostname host)} @ ${image-pathname}")
   (let ((chroot (ensure-directory-pathname
                  (strcat (unix-namestring image-pathname) ".chroot")))
         (volumes
           (loop
             with found
-            for volume in (getf (hostattrs (preprocess-host host)) :volumes)
+            for volume in (get-hostattrs :volumes (preprocess-host host))
             for physical-disk-p = (subtypep (type-of volume) 'physical-disk)
             if (and physical-disk-p (not found)
                     (slot-boundp volume 'volume-contents))
@@ -891,9 +951,38 @@ Do not apply in DEFHOST.  Apply with DEPLOY-THESE/HOSTDEPLOY-THESE."
   (:desc "Host volumes created")
   (:apply (create-volumes-and-contents (get-hostattrs :volumes))))
 
+;; TODO Possibly we want (a version of) this to not fail, but just do nothing,
+;; if the relevant volume groups etc. are inactive?
+(defproplist host-logical-volumes-exist :lisp ()
+  "Create missing logical volumes, like LVM logical volumes and BTRFS
+subvolumes, as specified by DISK:HAS-VOLUMES.  Does not delete or overwrite
+anything, aside from editing /etc/fstab in some cases.  Intended to make it
+easy to add new logical volumes by just editing the volumes specification.
+
+For logical volumes containing instances of FILESYSTEM with a specified
+MOUNT-POINT, ensure there's an /etc/fstab entry and try to mount.
+
+Currently only handling of LVM logical volumes is implemented."
+  (:desc "Host logical volumes all exist")
+  (host-lvm-logical-volumes-exist))
+
 
 ;;;; Utilities
 
+(defun parse-volume-size (volume-size-specification)
+  (if (stringp volume-size-specification)
+      (multiple-value-bind (match groups)
+          (re:scan-to-strings #?/\A([0-9]+)([MGT])?\z/ volume-size-specification)
+        (unless match
+          (simple-program-error
+           "~A is not a valid volume size." volume-size-specification))
+        (* (parse-integer (elt groups 0))
+           (eswitch ((elt groups 1) :test #'string=)
+             ("M" 1)
+             ("G" 1024)
+             ("T" 1048576))))
+      volume-size-specification))
+
 (defmacro volumes (&body volume-specifications)
   "Return a list of instances of VOLUME, one for each element of
 VOLUME-SPECIFICATIONS.  Each of VOLUME-SPECIFICATIONS is an (unquoted) list of
@@ -927,12 +1016,10 @@ Example usage:
         (luks-container
          (lvm-physical-volume
           :volume-group \"vg_melete\"))))))
-    (lvm-volume-group
-     :volume-label \"vg_melete\"
-     ((lvm-logical-volume
+    (lvm-logical-volume
+       :volume-group \"vg_melete\"
        :volume-label \"lv_melete_root\"
-       (ext4-filesystem
-        :mount-point #P\"/\")))))"
+       (ext4-filesystem :mount-point #P\"/\")))"
   (labels
       ((parse (spec)
          (unless (listp spec)
@@ -942,20 +1029,9 @@ Example usage:
                   (if contentsp (butlast (cdr spec)) (cdr spec)))
                 (contents (and contentsp (lastcar (cdr spec)))))
            (when (loop for key on initargs by #'cddr
-                         thereis (and (eql (car key) :volume-size)
-                                      (stringp (cadr key))))
-             (let ((input (getf initargs :volume-size)))
-               (multiple-value-bind (match groups)
-                   (re:scan-to-strings #?/\A([0-9]+)([MGT])?\z/ input)
-                 (unless match
-                   (simple-program-error
-                    "~A is not a valid volume size." input))
-                 (setf (getf initargs :volume-size)
-                       (* (parse-integer (elt groups 0))
-                          (eswitch ((elt groups 1) :test #'string=)
-                            ("M" 1)
-                            ("G" 1024)
-                            ("T" 1048576)))))))
+                         thereis (and (eql (car key) :volume-size)))
+             (setf (getf initargs :volume-size)
+                   `(parse-volume-size ,(getf initargs :volume-size))))
            (when (and contents (not (listp contents)))
              (simple-program-error "~A is not a list." contents))
            `(make-instance
