@@ -98,7 +98,8 @@ CONTENT can be a list of lines or a single string."
    (containing-directory-exists destination)
    (maybe-writefile-data destination iden1 iden2 :mode #o600)))
 
-(defprop host-secret-uploaded :posix (destination)
+(defprop host-secret-uploaded :posix
+    (destination &aux (destination (unix-namestring destination)))
   (:hostattrs
    (require-data (get-hostname) destination))
   (:apply
@@ -438,25 +439,23 @@ not NO-SOURCE and the corresponding member of ENTRIES is STRING= to either
 NO-SOURCE or \"PLACEHOLDER\", use the existing field value."
   (let ((unknown (list no-source "PLACEHOLDER"))
         (pending (make-hash-table :test #'equal)))
-    (flet ((fields (entry)
-             (remove "" (split-string entry) :test #'string=)))
-      (dolist (entry entries)
-        (setf (gethash (nth target (fields entry)) pending) entry))
-      (map-file-lines
-       file
-       (lambda (lines)
-         (loop for line in lines
-               for line-fields = (fields line)
-               for line-source = (nth source line-fields)
-               and line-target = (nth target line-fields)
-               for entry = (when-let* ((entry (gethash line-target pending))
-                                       (fields (fields entry)))
-                             (and (member (nth source fields)
-                                          unknown :test #'string=)
-                                  (not (string= line-source no-source))
-                                  (setf (nth source fields) line-source)
-                                  (format nil "~{~A~^ ~}" fields)))
-               if entry
-                 collect it into accum and do (remhash line-target pending)
-               else collect line into accum
-               finally (return (nconc accum (hash-table-values pending)))))))))
+    (dolist (entry entries)
+      (setf (gethash (nth target (words entry)) pending) entry))
+    (map-file-lines
+     file
+     (lambda (lines)
+       (loop for line in lines
+             for line-fields = (words line)
+             for line-source = (nth source line-fields)
+             and line-target = (nth target line-fields)
+             for entry = (when-let* ((entry (gethash line-target pending))
+                                     (fields (words entry)))
+                           (when (and (member (nth source fields)
+                                              unknown :test #'string=)
+                                      (not (string= line-source no-source)))
+                             (setf (nth source fields) line-source))
+                           (format nil "~{~A~^ ~}" fields))
+             if entry
+               collect it into accum and do (remhash line-target pending)
+             else collect line into accum
+             finally (return (nconc accum (hash-table-values pending))))))))
